@@ -20,13 +20,13 @@ Contact: yuvraaj@gmail.com
 */
 
 #include "TpCalloutInitiator.h"
-#include <TelepathyQt4/PendingChannelRequest>
-#include <TelepathyQt4/Connection>
 
 #if defined(Q_WS_MAEMO_5) || defined(MEEGO_HARMATTAN)
-#define CSD_SERVICE         "com.nokia.csd"
-#define CSD_CALL_PATH		"/com/nokia/csd/call"
-#define CSD_CALL_INTERFACE	"com.nokia.csd.Call"
+#define CSD_SERVICE                 "com.nokia.csd"
+#define CSD_CALL_PATH               "/com/nokia/csd/call"
+#define CSD_CALL_INSTANCE_PATH      "/com/nokia/csd/call/1"
+#define CSD_CALL_INTERFACE          "com.nokia.csd.Call"
+#define CSD_CALL_INSTANCE_INTERFACE	"com.nokia.csd.Call.Instance"
 #endif
 
 TpCalloutInitiator::TpCalloutInitiator (Tp::AccountPtr act, QObject *parent)
@@ -34,6 +34,7 @@ TpCalloutInitiator::TpCalloutInitiator (Tp::AccountPtr act, QObject *parent)
 , account (act)
 , strActCmName("undefined")
 , strSelfNumber("undefined")
+, systemBus(QDBusConnection::systemBus())
 , bIsSpirit (false)
 , channel (NULL)
 , toneOn(false)
@@ -71,6 +72,7 @@ TpCalloutInitiator::TpCalloutInitiator (Tp::AccountPtr act, QObject *parent)
 
     Tp::ConnectionPtr connection = account->connection();
     onConnectionChanged (connection);
+
 }//TpCalloutInitiator::TpCalloutInitiator
 
 void
@@ -185,15 +187,16 @@ TpCalloutInitiator::initiateCall (const QString &strDestination,
 
 #if USE_RAW_CHANNEL_METHOD
     QVariantMap request;
-    request.insert(TELEPATHY_INTERFACE_CHANNEL ".ChannelType",
-                   TELEPATHY_INTERFACE_CHANNEL_TYPE_STREAMED_MEDIA);
-    request.insert(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType",
+    request.insert(QString(TPQT_IFACE_CHANNEL) + ".ChannelType",
+                   TPQT_CHANNEL_TYPE_STREAMED_MEDIA);
+    request.insert(QString(TPQT_IFACE_CHANNEL) + ".TargetHandleType",
                    (uint) Tp::HandleTypeContact);
-    request.insert(TELEPATHY_INTERFACE_CHANNEL ".TargetID",
+    request.insert(QString(TPQT_IFACE_CHANNEL) + ".TargetID",
                    strDestination);
+
     if (!bIsSpirit) {
-        request.insert(TELEPATHY_INTERFACE_CHANNEL
-                       ".Type.StreamedMedia.InitialAudio",
+        request.insert(QString(TPQT_IFACE_CHANNEL) +
+                        ".Type.StreamedMedia.InitialAudio",
                        true);
     }
 
@@ -208,17 +211,21 @@ TpCalloutInitiator::initiateCall (const QString &strDestination,
 
     rv =
     connect (pReq, SIGNAL (finished (Tp::PendingOperation*)),
-            this, SLOT   (onChannelReady (Tp::PendingOperation*)));
+             this, SLOT   (onChannelReady (Tp::PendingOperation*)));
     if (!rv) {
         Q_WARN("Failed to connect to call ready signal!!");
         Q_ASSERT(0 == "Failed to connect to call ready signal!!");
     }
+
+    startMonitoringCall();
 }//TpCalloutInitiator::initiateCall
 
 void
 TpCalloutInitiator::onChannelReady (Tp::PendingOperation *op)
 {
     bool bSuccess = false;
+    Tp::ChannelPtr ch1;
+
     do { // Begin cleanup block (not a loop)
         if (op->isError ()) {
             Q_WARN ("Channel could not become ready");
@@ -229,10 +236,17 @@ TpCalloutInitiator::onChannelReady (Tp::PendingOperation *op)
         bSuccess = true;
 
         Tp::PendingChannelRequest *pReq = (Tp::PendingChannelRequest *) op;
+        if (NULL == pReq->channelRequest().constData ()) {
+            Q_WARN("Channel request is NULL");
+            break;
+        }
+
+        ch1 = pReq->channelRequest().constData ()->channel ();
+
 #if USE_RAW_CHANNEL_METHOD
-        channel = Tp::ChannelPtr::staticCast (pReq->object ());
+        channel = ch1;
 #else
-        channel = Tp::StreamedMediaChannelPtr::staticCast (pReq->object ());
+        channel = Tp::StreamedMediaChannelPtr::dynamicCast(ch1);
         if (channel->awaitingLocalAnswer()) {
             channel->acceptCall();
         }
@@ -284,27 +298,22 @@ TpCalloutInitiator::sendDTMF (const QString &strTones)
     do { // Begin cleanup block (not a loop)
 #if defined(Q_WS_MAEMO_5) || defined(MEEGO_HARMATTAN)
         if ("ring" == account->cmName ()) {
-            QList<QVariant> argsToSend;
-            argsToSend.append(strTones);
-            argsToSend.append(0);
-            QDBusConnection systemBus = QDBusConnection::systemBus();
-            QDBusMessage dbusMethodCall =
-                    QDBusMessage::createMethodCall(CSD_SERVICE,
-                                                   CSD_CALL_PATH,
-                                                   CSD_CALL_INTERFACE,
-                                                   QString("SendDTMF"));
-            dbusMethodCall.setArguments(argsToSend);
-            rv = systemBus.send(dbusMethodCall);
-            if (!rv) {
-                Q_WARN ("Dbus method call to send DTMF failed.");
+            if (bMaemoAudioConnected) {
+                sendMaemoDTMF(strTones);
+            } else {
+                Q_DEBUG("Audio not yet connected.");
+                remainingTones = strTones;
             }
-
-            Q_DEBUG("CSD version of DTMF requested");
-            break;
+        } else {
+            Q_DEBUG("Not ring, so cannot send DTMF...");
         }
+        break;
+
 #elif !USE_RAW_CHANNEL_METHOD
+
         remainingTones = strTones;
         onDtmfNextTone ();
+        break;
 
 #elif USE_DTMF_INTERFACE_1
         if ((NULL == channel) || (channel.isNull ())) {
@@ -351,8 +360,11 @@ TpCalloutInitiator::sendDTMF (const QString &strTones)
 
         Q_DEBUG("DTMF tones sent");
         rv = true;
+        break;
 #else
+        Q_WARN("DTMF tones not supported");
         Q_UNUSED(strTones);
+        break;
 #endif
     } while (0); // End cleanup block (not a loop)
 
@@ -384,6 +396,11 @@ TpCalloutInitiator::onDtmfNextTone()
             channel->streamsForType(Tp::MediaStreamTypeAudio);
         if (streams.isEmpty ()) {
             Q_WARN("No audio streams??");
+            streams = channel->streams();
+        }
+
+        if (streams.isEmpty ()) {
+            Q_WARN("No streams at all???");
             break;
         }
 
@@ -462,3 +479,129 @@ TpCalloutInitiator::onDtmfNextTone()
 #endif
     } while (0); // End cleanup block (not a loop)
 }//TpCalloutInitiator::onDtmfNextTone
+
+void
+TpCalloutInitiator::startMonitoringCall()
+{
+    bool rv;
+
+    do { // Begin cleanup block (not a loop)
+        rv = systemBus.connect(QString(""),
+                               CSD_CALL_INSTANCE_PATH,
+                               CSD_CALL_INSTANCE_INTERFACE,
+                               QString("AudioConnect"), this,
+                               SLOT(readyToSendDTMF(const QDBusMessage&)));
+        if (!rv) {
+            Q_WARN(QString("Failed to connect to AudioConnect"));
+            break;
+        }
+
+        rv = systemBus.connect(QString(""),
+                               CSD_CALL_INSTANCE_PATH,
+                               CSD_CALL_INSTANCE_INTERFACE,
+                               QString("StoppedDTMF"), this,
+                               SLOT(allDTMFDone(const QDBusMessage&)));
+        if (!rv) {
+            Q_WARN(QString("Failed to connect to StoppedDTMF"));
+            break;
+        }
+
+        rv = systemBus.connect(QString(""),
+                               CSD_CALL_INSTANCE_PATH,
+                               CSD_CALL_INSTANCE_INTERFACE,
+                               QString("Terminated"), this,
+                               SLOT(stopMonitoringCall()));
+        if (!rv) {
+            Q_WARN(QString("Failed to connect to Terminated"));
+            break;
+        }
+
+        bMaemoAudioConnected = false;
+        Q_DEBUG("Start monitoring");
+    } while (0); // End cleanup block (not a loop)
+
+    if (!rv) {
+        stopMonitoringCall();
+    }
+}//TpCalloutInitiator::startMonitoringCall
+
+void
+TpCalloutInitiator::stopMonitoringCall()
+{
+    bool rv;
+
+    Q_DEBUG("Stop monotioring");
+
+    rv = systemBus.disconnect(QString(""),
+                              CSD_CALL_INSTANCE_PATH,
+                              CSD_CALL_INSTANCE_INTERFACE,
+                              QString("Terminated"), this,
+                              SLOT(stopMonitoringCall()));
+    if (!rv) {
+        Q_WARN(QString("Failed to disconnect from Terminated"));
+    }
+
+    rv = systemBus.disconnect(QString(""),
+                              CSD_CALL_INSTANCE_PATH,
+                              CSD_CALL_INSTANCE_INTERFACE,
+                              QString("StoppedDTMF"), this,
+                              SLOT(displayDTMFConfirmation(const QDBusMessage&)));
+     if (!rv) {
+         Q_WARN(QString("Failed to disconnect from StoppedDTMF"));
+     }
+
+     rv = systemBus.disconnect(QString(""),
+                               CSD_CALL_INSTANCE_PATH,
+                               CSD_CALL_INSTANCE_INTERFACE,
+                               QString("AudioConnect"), this,
+                               SLOT(sendNumberAsDTMFCode(const QDBusMessage&)));
+     if (!rv) {
+        Q_WARN(QString("Failed to disconnect from AudioConnect"));
+     }
+}//TpCalloutInitiator::stopMonitoringCall
+
+void
+TpCalloutInitiator::readyToSendDTMF(const QDBusMessage &msg)
+{
+    QList<QVariant> listArguments = msg.arguments();
+    bMaemoAudioConnected = listArguments.first().toBool();
+
+    if (bMaemoAudioConnected && !remainingTones.isEmpty()) {
+        sendMaemoDTMF(remainingTones);
+        remainingTones.clear();
+    } else {
+        Q_DEBUG("Audio not yet connected.");
+    }
+}//TpCalloutInitiator::readyToSendDTMF
+
+void
+TpCalloutInitiator::allDTMFDone(const QDBusMessage & /*msg*/)
+{
+    Q_DEBUG("DTMF sent.");
+    stopMonitoringCall();
+}//TpCalloutInitiator::allDTMFDone
+
+void
+TpCalloutInitiator::sendMaemoDTMF(const QString &strTones)
+{
+    if (strTones.isEmpty())  {
+        Q_WARN("No DTMF tones to send");
+        return;
+    }
+
+    QList<QVariant> argsToSend;
+    argsToSend.append(strTones);
+//    argsToSend.append(0);
+    QDBusMessage dbusMethodCall =
+    QDBusMessage::createMethodCall(CSD_SERVICE, CSD_CALL_PATH,
+                                   CSD_CALL_INTERFACE,
+                                   QString("SendDTMF"));
+    dbusMethodCall.setArguments(argsToSend);
+    bool rv = systemBus.send(dbusMethodCall);
+    if (!rv) {
+        Q_WARN ("CSD method call to send DTMF failed.");
+    } else {
+        Q_DEBUG("CSD version of DTMF requested");
+    }
+}//TpCalloutInitiator::sendMaemoDTMF
+
